import os
from cv2 import cv2
import threading
import glob
import sys
import time
import configparser

# 定義計算輪廓相似度的函數
def calculate_similarity(refpath, dutpath, idx, similarities,ssim):
    ref_img = cv2.imread(refpath)
    dut_img = cv2.imread(dutpath)
    ref_gray = cv2.cvtColor(ref_img, cv2.COLOR_BGR2GRAY)
    dut_gray = cv2.cvtColor(dut_img, cv2.COLOR_BGR2GRAY)
    # 使用Canny算子提取图像轮廓
    edges1 = cv2.Canny(ref_gray, 100, 255)
    edges2 = cv2.Canny(dut_gray, 100, 255)

    # 提取轮廓
    contours1, hierarchy1 = cv2.findContours(
        edges1, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    contours2, hierarchy2 = cv2.findContours(
        edges2, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    # 绘制轮廓
    cv2.drawContours(ref_img, contours1, -1, (0, 0, 255), 1)
    cv2.drawContours(dut_img, contours2, -1, (0, 0, 255), 1)

    # 计算轮廓相似度

    # 比较两个图像的轮廓
    similarities[idx] = cv2.matchShapes(
        edges1, edges2, cv2.CONTOURS_MATCH_I1, 0)
    similarities[idx] = 1 - similarities[idx]
    if similarities[idx] < 0:
        similarities[idx] = 0

    if ssim_test == 1:
        ssim[idx] = cv2.matchTemplate(ref_gray, dut_gray, cv2.TM_CCORR_NORMED)
        print("ssim(-1..+1): ", ssim[idx].max())
        
    if show_result_image > 0:
        # 显示图像
        # 擴展
        # 以img2 的size 當參考 縮放
        dut_img = cv2.resize(dut_img, (ref_img.shape[1], ref_img.shape[0]))
        # 水平合并
        merged = cv2.hconcat([dut_img, ref_img])
        merged = cv2.copyMakeBorder(
            merged, 000, 40, 0, 0, cv2.BORDER_CONSTANT, value=[0, 0, 0])
        # 显示合并后的图像
        # 字
        text = 'Similarity: {:8.8f}'.format(similarities[idx])
        color = (255, 255, 0)  # 文本颜色  B G R
        thickness = 2  # 文本线宽
        org = (10, merged.shape[0]-10)
        font = cv2.FONT_HERSHEY_SIMPLEX  # 字体类型
        font_scale = 1  # 字体大小
        # 匯入字
        merged = cv2.putText(merged, text, org, font,font_scale, color, thickness)
        if ssim_test == 1:  # ssim
            merged = cv2.copyMakeBorder(merged, 000, 40, 0, 0, cv2.BORDER_CONSTANT, value=[0, 0, 0])
            text = 'SSIM: ' + str(ssim[idx].max())
            color = (255, 255, 0)  # 文本颜色  B G R
            thickness = 2  # 文本线宽
            org = (10, merged.shape[0] - 12)
            font = cv2.FONT_HERSHEY_SIMPLEX  # 字体类型
            font_scale = 1  # 字体大小
            # 匯入字
            merged = cv2.putText(merged, text, org, font, font_scale, color, thickness)
        

        # 檔案名稱
        image_file_name = result_image_path + \
            "result_" + os.path.basename(ref_path)
        # cv2.imwrite("output.jpg", merged)
        print("save file name: ", image_file_name)
        cv2.imwrite(image_file_name, merged)
        # 顯示影像 
        cv2.imshow(ref_path, merged)
        cv2.waitKey(show_result_image)
        # cv2.destroyAllWindows()




print("Num of Parameter:", len(sys.argv))
'''
show or not show result image :   s:0 (now show)   s:1000  (show result image 1 second per picture)
example :
    Contour_Similarity_inf.exe s:100 P:d:/aaa/
    Contour_Similarity_inf.exe s:100 P:./aaa/

'''
start_time = time.time()

show_result_image = 1
result_image_path = ''
ssim_test = 1
n = len(sys.argv)
for p0 in range(1, n):   # skip first exe name
    s = sys.argv[p0]
    print(s)
    colon_index = s.index(':')
    s0 = s[:colon_index]
    if s0 == "s" or s0 == 'S':  # show,save image or not
        i0 = int(s[colon_index + 1:])
        print("s parameter found")
        print("t=", i0, "ms")
        show_result_image = i0
    if s0 == "ssim" or s0 == 'SSIM':
        ssim_test = int(s[colon_index + 1:])
        print("SSIM parameter found")
        print("ssim_test: ", ssim_test)   # 0 no test, 1:test ssim
    if s0 == "P" or s0 == 'p':   # path
        result_image_path = s[colon_index + 1:]
        print("P parameter found")
        print("Result_image_path: ", result_image_path)
        if not os.path.exists(result_image_path):
            # Create the directory if it does not exist
            os.makedirs(result_image_path)
            print(f"Created directory: {result_image_path}")
        else:
            print(f"Directory already exists: {result_image_path}")


# show_result_image = 1  #  0  no show, none >0 :show n mS
# 設定目錄路徑

#dut_path = './DUT/'  # dut 圖片位置
dir_path_0 = os.path.dirname(os.path.dirname(dut_path))
if not os.path.exists(dut_path):
    # Create the directory if it does not exist
    os.makedirs(dut_path)
    print(f"Created directory: {dut_path}")
else:
    print(f"Directory already exists: {dut_path}")

#ref_path = parent_dir = dir_path_0 + "/REF/"  # Golden smaple 圖片位置

# 使用 glob 模塊列出目錄下所有 JPG 檔案
jpg_files = glob.glob(os.path.join(dut_path, '*.[jJ][pP][gG]'))+ glob.glob(os.path.join(dut_path, '*.[bB][mM][pP]'))
inf_file_name = dut_path + "Contour_Similarity.inf"

write_config = configparser.ConfigParser()

if os.path.exists(inf_file_name):
    write_config.read(inf_file_name)
else:
    write_config.add_section("Contour_Similarity")

write_config.set("Contour_Similarity", "Running_status", str(1))
cfgfile = open(inf_file_name, 'w')
write_config.write(cfgfile)
cfgfile.close()
valid_DUT_files_cnt = 0
valid_REF_files_cnt = 0

# 設置相似度值列表
similarities = [0] * len(jpg_files)
ssim = [0] * len(jpg_files)
# 建立和啟動執行緒
threads = []
thread_count = len(jpg_files)
for i in range(thread_count):
    # 逐一印出所有 JPG 檔案的檔名
    write_config.set("Contour_Similarity", "%s" % os.path.basename(jpg_files[i]), str(0))
    if os.path.exists(jpg_files[i]):
        print("DUT file: ", jpg_files[i])
        valid_DUT_files_cnt = valid_DUT_files_cnt + 1
    else:
        print("DUT file: ", jpg_files[i], " not exist")
        continue
    jpg_file_0 = os.path.basename(jpg_files[i])
    jpg_file_0 = ref_path + jpg_file_0

    if os.path.exists(jpg_file_0):
        print("Ref file: ", jpg_file_0)
        valid_REF_files_cnt = valid_REF_files_cnt + 1
    else:
        print("Ref file: ", jpg_file_0, " not exist")
        continue

    
    thread = threading.Thread(target=calculate_similarity, args=(jpg_files[i], jpg_file_0, i, similarities,ssim))
    thread.start()
    threads.append(thread)



# 等待所有執行緒完成
for thread in threads:
    thread.join()

for i in range(len(similarities)):
    print('Similarity between of %s %.9f' %(jpg_files[i], similarities[i]))
    
    # 寫INF
    write_config.set("Contour_Similarity", "%s" % os.path.basename(jpg_files[i]), str( similarities[i]))
    if ssim_test == 1:
        write_config.set("Contour_Similarity", "ssim_%s" % os.path.basename(jpg_files[i]), str(ssim[i].max()))



write_config.set("Contour_Similarity", "Running_status", str(0))
write_config.set("Contour_Similarity", "valid_DUT_files_cnt", str(valid_DUT_files_cnt))
write_config.set("Contour_Similarity", "valid_REF_files_cnt", str(valid_REF_files_cnt))
end_time = time.time()
running_time = end_time - start_time
write_config.set("Contour_Similarity", "testing time", (f"Running time: {running_time} seconds"))
print('running_time: ',running_time)
cfgfile = open(inf_file_name, 'w')
write_config.write(cfgfile)
cfgfile.close()

#cv2.waitKey(show_result_image)
cv2.destroyAllWindows()
